/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QUACKLE_GAMEPARAMETERS_H
#define QUACKLE_GAMEPARAMETERS_H

namespace Quackle
{

class GameParameters
{
public:
	GameParameters();

	int minimumTilesForExchange() const;
	void setMinimumTilesForExchange(int minimumTilesForExchange);

	// negative number -> no limit
	int numberOfScorelessTurnsThatEndsGame() const;
	void setNumberOfScorelessTurnsThatEndsGame(int numberOfScorelessTurnsThatEndsGame);

	int bingoBonus() const;
	void setBingoBonus(int bingoBonus);

	int rackSize() const;
	void setRackSize(int rackSize);

protected:
	int m_minimumTilesForExchange;
	int m_numberOfScorelessTurnsThatEndsGame;
	int m_bingoBonus;
	int m_rackSize;
};

inline int GameParameters::minimumTilesForExchange() const
{
	return m_minimumTilesForExchange;
}

inline int GameParameters::numberOfScorelessTurnsThatEndsGame() const
{
	return m_numberOfScorelessTurnsThatEndsGame;
}

inline void GameParameters::setNumberOfScorelessTurnsThatEndsGame(int numberOfScorelessTurnsThatEndsGame)
{
	m_numberOfScorelessTurnsThatEndsGame = numberOfScorelessTurnsThatEndsGame;
}

inline int GameParameters::bingoBonus() const
{
	return m_bingoBonus;
}

inline int GameParameters::rackSize() const
{
	return m_rackSize;
}

inline void GameParameters::setMinimumTilesForExchange(int minimumTilesForExchange)
{
	m_minimumTilesForExchange = minimumTilesForExchange;
}

inline void GameParameters::setBingoBonus(int bingoBonus)
{
	m_bingoBonus = bingoBonus;
}

inline void GameParameters::setRackSize(int rackSize)
{
	m_rackSize = rackSize;
}

class EnglishParameters : public GameParameters
{
public:
	EnglishParameters();
};

}

#endif
