/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QUACKER_BOARDSETUP_H
#define QUACKER_BOARDSETUP_H

#include "graphicalboard.h"

class BoardSetupFrame;

class BoardSetup : public View
{
Q_OBJECT

public:
	BoardSetup(QWidget *parent = 0);
	~BoardSetup() {};

	BoardSetupFrame *boardFrame() { return m_boardFrame; }

protected slots:
	virtual void expandToFullWidth();
	virtual void resizeEvent(QResizeEvent *event);

private:
	BoardSetupFrame *m_boardFrame;
	QWidget *m_boardWrapper;
	QVBoxLayout *m_vlayout;
};

class BoardSetupFrame : public GraphicalBoardFrame
{
Q_OBJECT

public:
	BoardSetupFrame(QWidget *parent = 0);
	~BoardSetupFrame();
	void setBoard(const Quackle::Board &board);
	void setSymmetry(bool horizontal, bool vertical);
	void parametersChanged();

public slots:
	virtual void positionChanged(const Quackle::GamePosition & /* position */) {};

protected slots:
	virtual void tileClicked(const QSize &tileLocation, const QMouseEvent *event);

protected:
	virtual void keyPressEvent(QKeyEvent * /* e */) {};

	virtual bool wantMousePressEvent(const QMouseEvent *event) const;

private:
	bool horizontalSymmetry;
	bool verticalSymmetry;

	void setMultipliers(int row, int col, int word, int letter);
};

#endif
