/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QUACKER_BOARDSETUPDIALOG_H
#define QUACKER_BOARDSETUPDIALOG_H

#include <string>
#include <game.h>

#include <QWidget>
#include <QDialog>

using namespace std;

class QCheckBox;
class QComboBox;
class QLineEdit;
class QPushButton;

class BoardSetupFrame;

class BoardSetupDialog : public QDialog
{
Q_OBJECT

public:
	BoardSetupDialog(QWidget *parent = 0);
	~BoardSetupDialog();
	virtual void accept();
	virtual void reject();

protected slots:
	void parametersChanged(const QString &unused);
	void symmetryChanged();
	void undoAllChanges();

private:
	QCheckBox *m_horizontalSymmetry;
	QCheckBox *m_verticalSymmetry;
	
	QComboBox *m_horizontalDimension;
	QComboBox *m_verticalDimension;
	
	QLineEdit *m_boardName;
	
	QPushButton *m_saveChanges;
	QPushButton *m_cancel;
	QPushButton *m_undoAll;
	
	Quackle::Game m_game;
	BoardSetupFrame * m_boardFrame;

	QString m_originalName;
	
	string m_serializedOriginalBoard;

	QComboBox * constructDimensionComboBox(int defaultDimension);
	void initializeBoardName();
};

#endif
