/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QUACKLE_DICT_H
#define QUACKLE_DICT_H

#include <QStringList>

namespace Dict
{

class Extension
{
public:
	Extension();

	QString word;
	int playability;
	double probability;
	bool british;
};

typedef QList<Extension> ExtensionList;

class Word
{
public:
	Word();

	QString word;
	int playability;
	double probability;
	bool british;

	ExtensionList frontExtensions;
	ExtensionList backExtensions;

	static ExtensionList extensionsByLength(int length, const ExtensionList &list);

	static void sortWordListByPlayability(QList<Word> &list)
	{
		qSort(list);
	}
};

class WordList : public QList<Word>
{
public:
	WordList();
	~WordList();

	enum SortType { Playability, Length, LengthLongestFirst, Probability };

	void setSortBy(SortType sortType);
	static SortType sortType;
};

class ThingToGetUpdatedDuringQueries
{
public:
	virtual void updateDuringQuery() = 0;
};

class Querier
{
public:
	enum QueryFlags { None = 0x0000, WithExtensions = 0x0001, CallUpdate = 0x0002, NoRequireAllLetters = 0x0004 };

	virtual void setThingToGetUpdatedDuringQueries(ThingToGetUpdatedDuringQueries * /* thing */) {};
	virtual WordList query(const QString &query, int flags = None) = 0;
	virtual QString arrangeLetters(const QString &letters, const QString &order) = 0;
	virtual QString alphagram(const QString &letters);
	virtual bool isLoaded() = 0;
};

}

// Returns true if word1 is less playable than word2;
// otherwise returns false.
bool operator<(const Dict::Word &word1, const Dict::Word &word2);

#endif
