/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QUACKLE_LISTER_H
#define QUACKLE_LISTER_H

#include <qdialog.h>
#include <qframe.h>

#include "dict.h"

class QCheckBox;
class QLabel;
class QLineEdit;
class QListWidget;
class QListWidgetItem;
class QPushButton;
class QSettings;
class QSpinBox;
class QVBoxLayout;

class Filter;

class ListerDialog : public QDialog
{
Q_OBJECT

public:
	enum ListerFlags { FullLister = 0x0000, IgnoreBritishness = 0x0001, ProbabilityInsteadOfPlayability = 0x0002, NothingToReturn = 0x0004 };

	ListerDialog(QWidget *parent, Dict::Querier *anagrammer, const QString &settingsGroup, const QString &appName = QString::null, int flags = FullLister);
	~ListerDialog();

	// use this for modal running! settingsGroup is like "letterbox"
	static QString run(QWidget *parent, Dict::Querier *anagrammer, const QString &settingsGroup, const QString &appName = QString::null, int flags = FullLister);

	Dict::WordList &wordList();
	void setWordList(Dict::WordList list);

	// map of alphagrams to words
	QMap<QString, Dict::WordList> anagramMap();

	QString writeList(bool alphagrams);
	QString filename() { return m_filename; }

	Dict::Querier *anagrammer() { return m_anagrammer; }
	int flags() { return m_flags; }

	static QSpinBox *makeSpinBox(int minimum, int maximum, int singleStep);

public slots:
	void setQuery(const QString &query);
	void queryGo();
	void setRemoveSowpods(bool on);

	// makes filter widget and save/load settings exactly once
	void showFilter(const QString &filterName);
	void showFilter(QListWidgetItem *item);

	void chooseFilename();
	void filenameChanged(const QString &);

	void clear();
	void openFile();

	void writeButtonClicked();
	void writeNormalButtonClicked();
	void studyButtonClicked();

	QString windowTitleWithAppName(const QString &windowTitle);

	void done();

protected:
	Dict::WordList m_wordList;
	Dict::Querier *m_anagrammer;
	QString m_settingsGroup;
	QString m_appName;
	int m_flags;

	void saveSettings();
	void loadSettings();

private:

	// left side
	QLineEdit *m_queryEdit;
	QPushButton *m_queryButton;
	QLabel *m_numResultsLabel;
	QCheckBox *m_sowpodsChecker;
	QCheckBox *m_buildChecker;

	// right side
	QListWidget *m_filtersBox;
	QVBoxLayout *m_filtersLayout;
	Filter *m_currentFilter;
	QPushButton *m_applyButton;

	// bottom
	QLineEdit *m_filenameEdit;
	QPushButton *m_writeButton;
	QPushButton *m_writeNormalButton;
	QPushButton *m_studyThisButton;
	QListWidget *m_listBox;

	QString m_filename;

	void populateListBox();
};

class Filter : public QFrame
{
Q_OBJECT

public:
	Filter(ListerDialog *dialog);

public slots:
	virtual void apply();
	virtual void saveSettings(QSettings *settings);
	virtual void loadSettings(QSettings *settings);

protected:
	ListerDialog *m_dialog;
	QVBoxLayout *m_vbox;
};

class RegexFilter : public Filter
{
Q_OBJECT

public:
	RegexFilter(ListerDialog *dialog);

public slots:
	virtual void apply();

private:
	QLineEdit *m_lineEdit;
};

class PlayabilityFilter : public Filter
{
Q_OBJECT

public:
	PlayabilityFilter(ListerDialog *dialog);

public slots:
	virtual void apply();
	virtual void saveSettings(QSettings *settings);
	virtual void loadSettings(QSettings *settings);

private:
	QSpinBox *m_minRankSpinner;
	QSpinBox *m_maxRankSpinner;
};

class NumAnagramsFilter : public Filter
{
Q_OBJECT

public:
	NumAnagramsFilter(ListerDialog *dialog);

public slots:
	virtual void apply();
	virtual void saveSettings(QSettings *settings);
	virtual void loadSettings(QSettings *settings);

private:
	QSpinBox *m_twlAnagramsSpinner;
	QSpinBox *m_oswOnlyAnagramsSpinner;
};

class KeepBritishFilter : public Filter
{
Q_OBJECT

public:
	KeepBritishFilter(ListerDialog *dialog);

public slots:
	virtual void apply();
};

#endif
