/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QUACKER_NEWGAME_H
#define QUACKER_NEWGAME_H

#include <QDialog>
#include <QMap>

#include <player.h>

class QComboBox;
class QGroupBox;
class QLineEdit;
class QTabWidget;
class QTreeWidget;
class QTreeWidgetItem;
class QPushButton;
class QSettings;
class PlayerTab;

class NewGameDialog : public QDialog
{
Q_OBJECT

public:
	NewGameDialog(QWidget *parent = 0);

	Quackle::PlayerList players() const;

public slots:
	void done();

protected:
	void saveSettings();
	void loadSettings();

private:
	QTabWidget *m_tabs;
	PlayerTab *m_playerTab;
};

class PlayerTab : public QWidget 
{
Q_OBJECT

public:
	PlayerTab(QWidget *parent = 0);

	Quackle::PlayerList players() const;

public slots:
	void saveSettings();

private slots:
	void addPlayer();
	void removePlayer();
	void selectionChanged();
	void playerEdited();
	void populatePlayers();

private:
	enum PlayerTreeRows { PlayerName = 0, PlayerType = 1 };

	bool hasSelection();
	Quackle::Player getLastPlayer();

	QTreeWidget *m_playersTreeWidget;
	QPushButton *m_addPlayerButton;
	QPushButton *m_removePlayerButton;

	QGroupBox *m_editGroup;
	QLineEdit *m_nameEdit;
	QComboBox *m_playerType;
	bool m_changingEditorManually;

	QMap<Quackle::Player, QTreeWidgetItem *> m_playerMap;

	void addPlayer(const Quackle::Player &player);
	void setItem(QTreeWidgetItem *item, const Quackle::Player &player);

	QString stringForTypes(int playerType, int computerType);
	void typesForString(const QString &typeString, int &playerType, int &computerType);
};

#endif
