/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <QtCore>

#include "datamanager.h"
#include "game.h"
#include "gcgio.h"
#include "trademarkedboards.h"

void testGCGIO();

int main()
{
	Quackle::DataManager dataManager;
	dataManager.setBoardParameters(new ScrabbleBoard());

	testGCGIO();
	return 0;
}

void testGCGIO()
{
	QuackleIO::GCGIO io;
	QFile file("capp.gcg");

	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
	{
		cerr << "Could not open gcg" << endl;
		return;
	}

	QTextStream in(&file);

	Quackle::Game *game = io.read(in, QuackleIO::Logania::BasicLoad);
	UVcout << game->history() << endl;

	UVcout << "Final scores: " << endl;
	Quackle::PlayerList players = game->currentPosition().endgameAdjustedScores();
	const Quackle::PlayerList::const_iterator end(players.end());
	for (Quackle::PlayerList::const_iterator it = players.begin(); it != end; ++it)
		UVcout << *it << endl;

	file.close();

	QFile outFile("my-capp.gcg");

	if (!outFile.open(QIODevice::WriteOnly | QIODevice::Text))
	{
		cerr << "Could not open gcg output file" << endl;
		return;
	}

	QTextStream out(&outFile);
	io.write(*game, out);

	outFile.close();

	delete game;
}

