/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QUACKLE_LOGANIA_H
#define QUACKLE_LOGANIA_H

#include "game.h"

class QTextStream;

namespace QuackleIO
{

// Abstract interfaces for game log input/output

class Logania
{
public:

	// if MaintainBoardPreparation is not set, boards will not be prepared
	// for analysis and you'll have to do this manually. If you set this flag
	// DataManager and a generator need to be set up.
	enum ReadFlags { BasicLoad = 0x0000, MaintainBoardPreparation = 0x0001 };

	// allocates a new Game, fills it up, and returns it.
	// Returned game is either 0, or a game with at least one position.
	// Boards of game are ready for analysis if there is a DataManager set
	// up; otherwise they won't have their crosses prepared
	virtual Quackle::Game *read(QTextStream &stream, int flags) = 0;

	virtual bool canRead(QTextStream &stream) const = 0;
	virtual void write(const Quackle::Game &game, QTextStream &stream) = 0;
	virtual QString filter() const = 0;
};

}

#endif
