/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <QtCore>

#include "gcgio.h"
#include "queenie.h"

using namespace QuackleIO;

Queenie *Queenie::m_self = 0;

Queenie *Queenie::self()
{
	if (m_self == 0)
		m_self = new Queenie();

	return m_self;
}

void Queenie::cleanUp()
{
	delete m_self;
	m_self = 0;
}

Queenie::Queenie()
{
	m_loganias.push_back(new GCGIO);

	m_filters.push_back("*");

	for (QList<Logania *>::const_iterator it = m_loganias.begin(); it != m_loganias.end(); ++it)
		m_filters.push_back((*it)->filter());
}

Queenie::~Queenie()
{
	while (!m_loganias.isEmpty())
		delete m_loganias.takeFirst();
}

Logania *Queenie::loganiaForFile(const QString &filename)
{
	for (QList<Logania *>::const_iterator it = m_loganias.begin(); it != m_loganias.end(); ++it)
	{
		QFile file(filename);
	
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		{
			cerr << "Could not open file" << endl;
			return 0;
		}

		QTextStream in(&file);

		if ((*it)->canRead(in))
			return (*it);
	}

	return 0;
}

const QStringList &Queenie::filters() const
{
	return m_filters;
}

Logania *Queenie::defaultLogania()
{
	return m_loganias.front();
}

