/*
 * Copyright (c) 2005-2006 Jason Katz-Brown and John O'Laughlin.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The names of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QUACKLE_RACK_H
#define QUACKLE_RACK_H

#include "alphabetparameters.h"

using namespace std;

namespace Quackle
{

class Move;

class Rack
{
public:
	// constructs a new empty rack
	Rack();

	// constructs a new rack containing letters in t
	Rack(const LetterString &tiles);

	// tiles like AEILNN?
	void setTiles(const LetterString &tiles);
	const LetterString &tiles() const;
	LetterString alphaTiles() const;

	bool empty() const;

	Rack& operator-=(const Move& M);

	// equivalent to operator-=(move.usedTiles())
	// and returns true if all tiles in used were found
	// in this rack and unloaded
	bool unload(const LetterString &used);

	// same as above but nonmutating
	bool contains(const LetterString &used) const;

	void shuffle();

	// sum of scores of letters on rack
	int score() const;

	UVString xml() const;
	UVString toString() const;

	static LetterString alphabetize(const LetterString &word);

private:
	LetterString m_tiles;
};

inline Rack::Rack()
{
}

inline Rack::Rack(const LetterString &tiles)
{
    setTiles(tiles);
}

inline void Rack::setTiles(const LetterString &tiles)
{
	m_tiles = tiles;
}

inline const LetterString &Rack::tiles() const
{
	return m_tiles;
}

inline bool Rack::empty() const
{
	return m_tiles.empty();
}

}

const Quackle::Rack operator-(const Quackle::Rack &rack, const Quackle::Move &move);
const Quackle::Rack operator-(const Quackle::Rack &rack1, const Quackle::Rack &rack);

UVOStream &operator<<(UVOStream &o, const Quackle::Rack &rack);

#endif
